#include <ArduinoRS485.h>
#include <ArduinoModbus.h>

#define LED_PIN 13

const int numCoils = 10;
const int numDiscreteInputs = 10;
const int numHoldingRegisters = 10;
const int numInputRegisters = 10;
long LED_CONTROL = 0;

void setup() {
  
  pinMode(LED_PIN, OUTPUT);
  
  Serial.begin(9600);
  while (!Serial);
  Serial.println("Arduino as a Modbus RTU Server");

  // start the Modbus RTU server, with slave ID of 1
  if (!ModbusRTUServer.begin(1, 9600)) 
  {
    Serial.println("Failed to start Modbus RTU Server!");
    while (1);
  }

  // configure coils at address 0x00
  ModbusRTUServer.configureCoils(0x00, numCoils);

  // configure discrete inputs at address 0x00
  ModbusRTUServer.configureDiscreteInputs(0x00, numDiscreteInputs);

  // configure holding registers at address 0x00
  ModbusRTUServer.configureHoldingRegisters(0x00, numHoldingRegisters);

  // configure input registers at address 0x00
  ModbusRTUServer.configureInputRegisters(0x00, numInputRegisters);
}

void loop() {
  // poll for Modbus RTU requests
  ModbusRTUServer.poll();

  // map the coil values to the discrete input values (what we write to the Arduino coils is echoed back into the discrete inputs on the PLC)
  for (int i = 0; i < numCoils; i++) {
    int coilValue = ModbusRTUServer.coilRead(i);
    ModbusRTUServer.discreteInputWrite(i, coilValue);
  }

  // map the holding register values to the input register values (what we write to the Arduino holding registers is echoed back into the input registers on the PLC)
  for (int i = 0; i < numHoldingRegisters; i++) {
    long holdingRegisterValue = ModbusRTUServer.holdingRegisterRead(i);
    long LED_CONTROL = ModbusRTUServer.holdingRegisterRead(0);
    ModbusRTUServer.inputRegisterWrite(i, holdingRegisterValue);
    
  // control the LED attached to pin 13 based on the value of the first holding register coming from the PLC (Modbus RTU master)    
    if (LED_CONTROL == 1) {
      digitalWrite(LED_PIN, HIGH);
     }
    else {
      digitalWrite(LED_PIN, LOW);
    }
  }
}
